#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::API::Tools
# state        public
#----------------------------------------------------------------------------------------
# description  Utilities used in different modules of GoogleSmartShopping cartridge.
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::API::Tools;
use base qw (Exporter);
use strict;

use DE_EPAGES::Core::API::TestParameter qw ( TestString );
use DE_EPAGES::Object::API::Factory qw (LoadRootObject);
use DE_EPAGES::ThirdPartyConfig::API::Log qw (Log3rdParty);

our @EXPORT_OK = qw (
    LogGoogleSmartShopping
);

#========================================================================================
# function     LogGoogleSmartShopping
#----------------------------------------------------------------------------------------
# syntax       LogGoogleSmartShopping($Name, $Debug, $Section);
#----------------------------------------------------------------------------------------
# example      LogGoogleSmartShopping('SendRequestHash', \%Hash, 'Send');
#----------------------------------------------------------------------------------------
# description  Logs a structure to the communication.log.
#----------------------------------------------------------------------------------------
# input        $Name | name of data | string
# input        $Debug | data to be logged | hash
# input        $Section | name of the section to be logged (optional - defaults to name of calling sub) | string
# input        $aFilter | (optional) filter (perl regular expression for
#                       s/$Filter[^\n]+\n/\n/gis) it can also be a CodeReference see Log3rdParty| ref.array.string
#========================================================================================
sub LogGoogleSmartShopping($$;$$) {
    my ($Name, $DebugStuff, $Section, $Filter) = @_;

    TestString('Section', $Section) if defined $Section;
    $Section //= (split(/::/, (caller(1))[3]))[-1];

    return unless LoadRootObject()->get('GoogleSmartShopping_Logging');
    Log3rdParty($Name, $DebugStuff, 'GoogleSmartShopping::'.$Section, $Filter);

    return;
}

1;