#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::t::unit::UI_GoogleSmartShopping
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for onboarding backlink redirection to the shop
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::t::unit::UI_GoogleSmartShopping;
use base DE_EPAGES::Core::API::Test;

use strict;

use DE_EPAGES::Test::Mock::Mockify qw ( WasCalled );
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;

use Test::Mockify;
use Test::Mockify::Matcher qw ( HashRef String );
use Test::Mockify::Sut;
use Test::More;


sub test {
    my $self = shift;

    $self->test_BacklinkGoogleSmartShoppingOnboardingRedirect();

    done_testing();
}

sub test_BacklinkGoogleSmartShoppingOnboardingRedirect {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $UrlBuilder = $self->_createUrlBuilder();
    my $SUT = $self->_createSUT($UrlBuilder);
    my $Servlet = $self->_createServlet();

    $SUT->BacklinkGoogleSmartShoppingOnboarding($Servlet);

    ok(
        WasCalled($UrlBuilder, 'buildRedirectToMboUrl'),
        $SubTestName . ' - Verify that MBO URL is built through buildRedirectToMboUrl method call.'
    );
    ok(
        WasCalled($SUT, '_RedirectToURL'),
        $SubTestName . ' - Verify that MBO shop redirection is done through _RedirectToURL call.'
    );

    return;
}

#----------------------------------------------------------------------------------------
# Mockery
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($UrlBuilder) = @_;

    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::GoogleSmartShopping::UI::GoogleSmartShopping');
    $MockifySut->mock('_RedirectToURL')
        ->when(String())
        ->thenReturnUndef();
    $MockifySut->mockStatic('DE_EPAGES::GoogleSmartShopping::API::GoogleSmartShoppingUrlBuilder::new')
        ->whenAny()
        ->thenReturn($UrlBuilder);

    return $MockifySut->getMockObject();
}

sub _createUrlBuilder {
    my $self = shift;

    my $Mockify = Test::Mockify->new('DE_EPAGES::GoogleSmartShopping::API::GoogleSmartShoppingUrlBuilder');
    $Mockify->mock('buildRedirectToMboUrl')
        ->when(HashRef())
        ->thenReturn('built-shop-mbo-url');

    return $Mockify->getMockObject();
}

sub _createServlet {
    my $self = shift;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($self->_createShop());
    $MockifyServlet->addMockWithReturnValue('form', $self->_createForm());

    return $MockifyServlet->getMockObject();
}

sub _createShop {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {});

    return $Mockify->getMockObject();
}

sub _createForm {
    my $self = shift;

    my $Mockify = Test::Mockify->new( 'DE_EPAGES::Presentation::API::Form');
    $Mockify->mock('queryValues')
        ->when()
        ->thenReturn({ 'status' => 'shop-mbo-url' });

    return $Mockify->getMockObject();
}


__PACKAGE__->runScript();