use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Cartridge::API::Table::Cartridge qw ( ExistsCartridgeByPackage );
use DE_EPAGES::Core::API::Script  qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Factory qw( LoadObject LoadObjectByPath );

sub Main {

    local $| = 1;

    my $ObjectID;
    my $Path;
    my $Password;
    my $StoreName;
    my $FileName;
    my $Help;

    GetOptions(
        'help'          => \$Help,
        'passwd=s'      => \$Password,
        'storename=s'   => \$StoreName,
        'path=s'        => \$Path,
        'objectid=s'    => \$ObjectID,
        'file=s'        => \$FileName,
    );
    usage() if $Help;
    usage() unless $FileName;

    unless(defined $StoreName and (defined $ObjectID or defined $Path)) {
        usage();
    }

    my $time1 = time;

    RunOnStore(
        Store       => $StoreName,
        DBPassword  => $Password,
        Sub         => sub  {
            unless (ExistsCartridgeByPackage('DE_EPAGES::ProductPortal')) {
                print "Store $StoreName: Cartridge ProductPortal is not installed.\n";
                return;
            }
            my $Object = defined $ObjectID ? LoadObject($ObjectID) : LoadObjectByPath($Path);
            $Object->exportPortal($FileName);
        },
    );
    my $time2 = time;
    printf "need %.3f seconds\n", $time2-$time1;
}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] flags objects

options:
    -passwd     database user password              ()
    -objectid   id of object                        ()
    -path       path to object                      ()
    -file       output file name                    ()
    -storename  name of store                       ()
flags:
    -help       show the command line options

Example:
    perl $0 -storename Store -path "/Shops/MyShop/PortalConfigs/MyPortal_de_DE" -file export.csv
    perl $0 -storename Store -objectid 123456 -file export.csv
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main,
);

1;
