#========================================================================================
# package      DE_EPAGES::Saferpay2::API::Requirement
# state        public
#----------------------------------------------------------------------------------------
# description  Provides methods in order to determine if the requirements 
#               for the usage of Saferpay2 are fullfilled.
#========================================================================================
package DE_EPAGES::Saferpay2::API::Requirement;

use DE_EPAGES::Saferpay2::API::Constants qw (
    REQUIRED_BILLING_ADDRESS_FIELDS_KLARNA
    PAYMENT_METHOD_KLARNA
);
use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# input        Shop | Shop object to check the requirements on | 
# return       $self | self | DE_EPAGES::Saferpay2::API::Requirement
#========================================================================================
sub new {
    my $class = shift;
    my ($Shop, $SplitGatewayMethod) = @_;
    my $self  = bless {
        'Shop'               => $Shop,
        'SplitGatewayMethod' => $SplitGatewayMethod
    }, $class;
    return $self;
}

#========================================================================================
# function     areMandatoryFieldsSet
# state        public
#----------------------------------------------------------------------------------------
# syntax       $AreMandatoryFieldsSet = $Requirement->areMandatoryFieldsSet();
#----------------------------------------------------------------------------------------
# description  Determines if all necessary billing address fields are set as mandatory.
#----------------------------------------------------------------------------------------
# return       True if all necessary fields are set as mandatory, 
#               otherwise False is returned. | boolean
#========================================================================================
sub areMandatoryFieldsSet {
    my $self = shift;
    my $Shop = $self->{'Shop'};
    my $SplitGatewayMethod = $self->{'SplitGatewayMethod'};
    my $MandatoryBillingAddressFields = $Shop->get('BillingAddressSetMandatoryFields');

    if($SplitGatewayMethod eq PAYMENT_METHOD_KLARNA) {
        foreach my $RequiredFieldName (REQUIRED_BILLING_ADDRESS_FIELDS_KLARNA) {
            unless ($MandatoryBillingAddressFields =~ /$RequiredFieldName/) {
                return 0;
            }
        }
    }
    return 1;
}

1;