#========================================================================================
# package      DE_EPAGES::Saferpay2::API::Tools
# state        public
#----------------------------------------------------------------------------------------
# description  Tooling for specific tasks
#========================================================================================
package DE_EPAGES::Saferpay2::API::Tools;
use base qw( Exporter );
use DE_EPAGES::Object::API::Table::Currency qw ( GetInfoCurrency );


use strict;

our @EXPORT_OK = qw (
    HandleTransactionResponse
    FormatFloatForSaferpay
);


#========================================================================================
# function     HandleTransactionResponse
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = HandleTransactionResponse($hResponse);
#----------------------------------------------------------------------------------------
# description  Checks if the response contains errors
#----------------------------------------------------------------------------------------
# input        $hResponse | hash
# return       $hResponse | hash
#========================================================================================
sub HandleTransactionResponse {
    my ($hResponse) = @_;

    $hResponse->{'HasErrorsInResponse'} = 0;

    if ( defined($hResponse->{'ErrorName'}) ) {
        $hResponse->{'Transaction'}->{'Id'} = $hResponse->{'TransactionId'} // 'NONE';
        $hResponse->{'Transaction'}->{'Status'} = 'FAILED';
        $hResponse->{'Transaction'}->{'ExternalStatus'} = $hResponse->{'ErrorName'};
        $hResponse->{'HasErrorsInResponse'} = 1;
    } else {
        $hResponse->{'Transaction'}->{'ExternalStatus'} = $hResponse->{'Transaction'}->{'Status'};
    }

    return $hResponse;
}

#========================================================================================
# function     FormatFloatForSaferpay
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FormattedFloatNumber = FormatFloatForSaferpay($FloatNumber, $currencyId);
#----------------------------------------------------------------------------------------
# description  It formats float number according to Saferpay
#----------------------------------------------------------------------------------------
# input        $FloatNumber | float number | float
# input        $currencyId | currency ID| string
# return       $FormattedFloatNumber | formatted float number string | string
#========================================================================================
sub FormatFloatForSaferpay {
    my ($amount, $currencyId) = @_;

    my $hCurrency = GetInfoCurrency($currencyId);
    my $price = sprintf('%.0f', $amount * 10 ** $hCurrency->{'MinorUnit'});
    return $price;
}

1;
