#========================================================================================
# package      DE_EPAGES::Saferpay2::t::unit::InitializePaymentPageMapper.t
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Saferpay2::API::InitializePaymentPageMapper
#========================================================================================
package DE_EPAGES::Saferpay2::t::unit::InitializePaymentPageMapper;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::Exception;
use Test::Mockify::Sut;
use Test::Mockify::Matcher qw ( String Number );
use Test::More;

use strict;

use constant SAFERPAY_API_VERSION => '1.21';

sub test {
    my $self = shift;

    $self->test_Map_Payer_OrderItems_RiskFactors();
    done_testing();
}


sub test_Map_Payer_OrderItems_RiskFactors {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];


    my $MockedOrder = $self->_createOrder();
    my $MockedLineItemContainer = $MockedOrder->get('LineItemContainer');

    my $hData = {};
    my $SUT;
    lives_ok( sub { $SUT = $self->_createSUT($hData, $MockedLineItemContainer) }, "$SubTestName - Service initialization works." );
    $SUT->MapPayer();
    $SUT->MapOrderItems();
    $SUT->MapRiskFactors();

    is_deeply(
        $SUT->GetData(),
        {
            'Payer' => {
                'BillingAddress' => {
                    'FirstName'   => 'Testperson-de',
                    'LastName'    => 'Approved',
                    'Email'       => 'test@epages.com',
                    'CountryCode' => 'de',
                }
            },
            'Order' => {
                'Items' => [
                    {
                        'DiscountAmount' => 0,
                        'Name' => 'Standardzustellung',
                        'Quantity' => 1,
                        'TaxAmount' => '0',
                        'TaxRate' => '1900',
                        'Type' => 'SHIPPINGFEE',
                        'UnitPrice' => '588'
                    },
                    {
                        'DiscountAmount' => 0,
                        'Name' => 'normal',
                        'Quantity' => 1,
                        'TaxAmount' => '0',
                        'TaxRate' => '1900',
                        'Type' => 'SALESTAX',
                        'UnitPrice' => '849'
                    },
                    {
                        'DiscountAmount' => '0',
                        'Name' => 'T-Shirt - olive',
                        'Quantity' => 1,
                        'TaxAmount' => '0',
                        'TaxRate' => '1900',
                        'Type' => 'PHYSICAL',
                        'UnitPrice' => '2513'
                    },
                    {
                        'DiscountAmount' => '200',
                        'Name' => 'T-Shirt- dark blue',
                        'Quantity' => 2,
                        'TaxAmount' => '0',
                        'TaxRate' => '1900',
                        'Type' => 'PHYSICAL',
                        'UnitPrice' => '1000'
                    },
                    {
                        'DiscountAmount' => '0',
                        'Name' => 'Discount10',
                        'Quantity' => 1,
                        'TaxAmount' => '0',
                        'TaxRate' => '0',
                        'Type' => 'GIFTCARD',
                        'UnitPrice' => '431'
                    }
                ]
            },
            'RiskFactors' => {
                'DeliveryType' => 'HOMEDELIVERY',
            },
        },
        "$SubTestName - Request body with Payer, Order Items, RiskFactors is ok."
    );
}



sub _createOrder {
    my $self = shift;

    my $hAttributes = {
        'LineItemContainer' => $self->_createLineItemContainer(),
        'Alias'             => '42',
        'BillingAddress'    => $self->_createBillingAddress(),
    };
    my $MockifyCustomerOrder = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::CustomerOrder', $hAttributes
    );

    return $MockifyCustomerOrder->getMockObject();
}

sub _createBasketOrderWithBillingAddress {
    my $self = shift;
    my ( $Alias )= @_;
    $Alias //= '44';

    my $hAttributes = {
        'LineItemContainer' => undef,
        'Shop'              => undef,
        'Alias'             => $Alias,
        'BillingAddress'    => $self->_createBillingAddress(),
    };
    my $MockifyCustomerOrder = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::CustomerOrder', $hAttributes
    );

    return $MockifyCustomerOrder->getMockObject();
}

# tests: MapPayer
sub _createBillingAddress {
    my $self = shift;

    my $hAttributes = {
        'ID' => 1234,
        'Country' => {'Code2' => 'DE'},
        'FirstName' => 'Testperson-de',
        'LastName' => 'Approved',
        'EMail' => 'test@epages.com',
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Address::API::Object::Address',
        $hAttributes
    );;
}

sub _createLineItemContainer {
    my $self = shift;

    my $MockedBasketOrOrder = $self->_createBasketOrderWithBillingAddress();
    my @ContainerLineItems = ();

    push(@ContainerLineItems, $self->_createLineItemShipping());
    push(@ContainerLineItems, $self->_createLineItemTax());
    push(@ContainerLineItems, $self->_createLineItemProduct1());
    push(@ContainerLineItems, $self->_createLineItemProduct2());
    push(@ContainerLineItems, $self->_createLineItemCoupon1());


    my $TotalAmount = 0;
    foreach my $LineItem (@ContainerLineItems) {
        $TotalAmount += $LineItem->get('LineItemPrice');
    }


    my $hAttributes = {
        'Payment'             => $self->_createLineItemPaymentSaferpay2(),
        'CurrencyID'          => 'EUR',
        'LanguageID'          => 1,
        'GrandTotal'          => $TotalAmount,
        'BasketOrOrder'       => $MockedBasketOrOrder,
        'HasDownloadProducts' => 0
    };
    my $MockifyContainer = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes
    );
    $MockifyContainer->addMockWithReturnValue('children', \@ContainerLineItems);

    return $MockifyContainer->getMockObject();
}

sub _createLineItemShipping {
    my $self = shift;

    my $hAttributes = {
        'Alias' => 'LineItemShipping_609A7290-03BC-4246-A57E-AC110004E400',
        'NameOrAlias'   => 'Standardzustellung',
        'BasePrice'     => undef,
        'LineItemPrice' => 5.88,
        'Quantity'      => 1,
        'TaxRate'       => 0.19,
        'TaxAmount'     => undef,
        'Discount'      => undef,
    };
    my $MockifyLineItemShipping = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::LineItemShipping', $hAttributes
    );
    $MockifyLineItemShipping->mock('instanceOf', sub {
        shift;
        my ($ClassType) = @_;
        return 0 unless ($ClassType =~ m/LineItemShipping/);
        return 1;
    });

    return $MockifyLineItemShipping->getMockObject();
}

sub _createLineItemTax {
    my $self = shift;

    my $hAttributes = {
        'Alias'         => 'LineItemTax_60A258A0-ACA4-0229-9770-AC1100040F7A',
        'NameOrAlias'   => 'normal',
        'BasePrice'     => 44.7,
        'LineItemPrice' => 0,
        'Quantity'      => undef,
        'TaxRate'       => 0.19,
        'TaxAmount'     => 8.49,
        'Discount'      => undef,
    };
    my $MockifyLineItemTax = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::LineItemTax', $hAttributes
    );
    $MockifyLineItemTax->mock('instanceOf', sub {
        shift;
        my ($ClassType) = @_;
        return 0 unless ($ClassType =~ m/LineItemTax/);
        return 1;
    });

    return $MockifyLineItemTax->getMockObject();
}

sub _createLineItemProduct1 {
    my $self = shift;

    my $hAttributes = {
        'Alias' => 'LineItemProduct_60A258A0-5036-F3EB-6554-AC1100040FC6',
        'NameOrAlias'               => 'T-Shirt - olive',
        'BasePrice'                 => 25.13,
        'LineItemPrice'             => 25.13,
        'Quantity'                  => 1,
        'IsLineItemDownloadProduct' => 0,
        'TaxRate'                   => 0.19,
        'TaxAmount'                 => undef,
        'Discount'                  => 0,
    };
    my $MockifyLineItemProduct = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Product::API::Object::LineItemProduct', $hAttributes
    );
    $MockifyLineItemProduct->mock('instanceOf', sub {
        shift;
        my ($ClassType) = @_;
        return 0 unless ($ClassType =~ m/LineItemProduct/);
        return 1;
    });

    return $MockifyLineItemProduct->getMockObject();
}

sub _createLineItemProduct2 {
    my $self = shift;

    my $hAttributes = {
        'Alias'                     => 'LineItemProduct_60A258F7-33DE-5334-AC51-AC1100040F46',
        'NameOrAlias'               => 'T-Shirt- dark blue',
        'BasePrice'                 => 10,
        'LineItemPrice'             => 18,
        'Quantity'                  => 2,
        'IsLineItemDownloadProduct' => 0,
        'TaxRate'                   => 0.19,
        'TaxAmount'                 => undef,
        'Discount'                  => 2,
    };
    my $MockifyLineItemProduct = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Product::API::Object::LineItemProduct', $hAttributes
    );
    $MockifyLineItemProduct->mock('instanceOf', sub {
        shift;
        my ($ClassType) = @_;
        return 0 unless ($ClassType =~ m/LineItemProduct/);
        return 1;
    });

    return $MockifyLineItemProduct->getMockObject();
}

sub _createLineItemCoupon1 {
    my $self = shift;

    my $hAttributes = {
        'Alias'         => 'LineItemCoupon_60A25973-43B5-62AA-60A6-AC1100040F2A',
        'NameOrAlias'   => 'Discount10',
        'BasePrice'     => 43.13,
        'LineItemPrice' => -4.31,
        'Quantity'      => 1,
        'TaxRate'       => undef,
        'TaxAmount'     => undef,
        'TaxAmount'     => undef,
        'Discount'      => undef,
    };
    my $MockifyLineItemProduct = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Product::API::Object::LineItemProduct', $hAttributes
    );
    $MockifyLineItemProduct->mock('instanceOf', sub {
        shift;
        my ($ClassType) = @_;
        return 0 unless ($ClassType =~ m/LineItemCoupon/);
        return 1;
    });

    return $MockifyLineItemProduct->getMockObject();
}

sub _createLineItemPaymentSaferpay2 {
    my $self = shift;

    my $hAttributes = {
        'PaymentMethod'     => $self->_createPaymentMethod(),
    };
    my $MockifyContainer = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Saferpay2::API::Object::LineItemPaymentSaferpay2', $hAttributes
    );

    return $MockifyContainer->getMockObject();
}


sub _createPaymentMethod {
    my $self = shift;

    my $hAttributes = {
        SplitGatewayMethod => 'Klarna'
    };
    my $MockifyPaymentMethod = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Saferpay2::API::Object::PaymentMethodSaferpay2', $hAttributes
    );

    return $MockifyPaymentMethod->getMockObject();
}



sub _createSUT {
    my $self = shift;
    my ( $hData, $LineItemContainer ) = @_;

    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::Saferpay2::API::InitializePaymentPageMapper', [$hData, $LineItemContainer]);
    $MockifySut->mockImported('DE_EPAGES::Saferpay2::API::Tools', 'FormatFloatForSaferpay')->when(Number(), String())->thenCall(
        sub{return ($_[0] * 100);});

    return $MockifySut->getMockObject();
}



#========================================================================================
# Main
#========================================================================================

__PACKAGE__->runScript();
