#========================================================================================
# package      DE_EPAGES::Saferpay2::t::unit::Requirement.t
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Saferpay2::API::Requirement
#========================================================================================
package DE_EPAGES::Saferpay2::t::unit::Requirement;
use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::Saferpay2::API::Constants qw (
    PAYMENT_METHOD_KLARNA
);
use DE_EPAGES::Saferpay2::API::Requirement qw ( areMandatoryFieldsSet );
use Test::Mockify::Sut;
use Test::More;
use Test::Exception;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use constant PAYMENT_METHOD_CREDIT_DEBIT => 'CreditDebit';
use strict;


sub test {
    my $self = shift;

    $self->test_MandatoryFieldsNotRequired_CreditDebit();
    $self->test_AreMandatoryFieldsSet_Klarna();
    $self->test_AreMandatoryFieldsNotSet_Klarna();
    done_testing();
}

sub test_MandatoryFieldsNotRequired_CreditDebit {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $SUT;
    my $Shop = $self->_createShop('');
    my $SplitGatewayMethod = PAYMENT_METHOD_CREDIT_DEBIT;
    lives_ok( sub { $SUT = $self->_createSUT($Shop, $SplitGatewayMethod) }, "$SubTestName - initialization works." );
    is($SUT->areMandatoryFieldsSet(), 1, "$SubTestName - mandatory fields are not required for CreditDebit.");
}

sub test_AreMandatoryFieldsSet_Klarna {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $SUT;
    my $Shop = $self->_createShop('FirstName,LastName,Street,Zipcode,City,CountryID,EMail');
    my $SplitGatewayMethod = PAYMENT_METHOD_KLARNA;
    lives_ok( sub { $SUT = $self->_createSUT($Shop, $SplitGatewayMethod) }, "$SubTestName - initialization works." );
    is($SUT->areMandatoryFieldsSet(), 1, "$SubTestName - mandatory fields are set for Klarna.");
}

sub test_AreMandatoryFieldsNotSet_Klarna {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $SUT;
    my $Shop = $self->_createShop('FirstName,LastName,Street,Zipcode,City,EMail');
    my $SplitGatewayMethod = PAYMENT_METHOD_KLARNA;
    lives_ok( sub { $SUT = $self->_createSUT($Shop, $SplitGatewayMethod) }, "$SubTestName - initialization works." );
    is($SUT->areMandatoryFieldsSet(), 0, "$SubTestName - mandatory fields are not set for Klarna.");
}

sub _createShop {
    my $self = shift;
    my ($MandatoryFields) = @_;

    my $hAttributes = {
        'BillingAddressSetMandatoryFields' => $MandatoryFields
    };

    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop', $hAttributes
    );
    return $MockifyShop->getMockObject();

}

sub _createSUT {
    my $self = shift;
    my ($Shop, $SplitGatewayMethod) = @_;

    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::Saferpay2::API::Requirement', [$Shop, $SplitGatewayMethod]);
    return $MockifySut->getMockObject();
}

__PACKAGE__->runScript();
