#========================================================================================
# package      DE_EPAGES::Saferpay2::t::unit::Tools.t
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Saferpay2::API::Tools
#========================================================================================
package DE_EPAGES::Saferpay2::t::unit::Tools;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Saferpay2::API::Tools qw ( HandleTransactionResponse );
use Test::Mockify::Sut;
use Test::Mockify::Matcher qw ( String );
use Test::More;

use strict;


sub test {
    my $self = shift;

    $self->test_HandleInitializeErrorResponse_WithError();
    $self->test_HandleInitializeErrorResponse_WithoutError();
    $self->test_HandleAssertErrorResponse_WithError();
    $self->test_HandleAssertErrorResponse_WithoutError();
    $self->test_FormatFloatForSaferpay();
    done_testing();
}

sub test_HandleInitializeErrorResponse_WithError {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hParam = {
        ErrorName => 'TRANSACTION_DECLINED'
    };

    my $hExpectedHandledResponse = {
        ErrorName => 'TRANSACTION_DECLINED',
        HasErrorsInResponse => 1,
        Transaction         => {
            Id => 'NONE',
            Status => 'FAILED',
            ExternalStatus => 'TRANSACTION_DECLINED',
        }
    };

    my $hHandledResponse = HandleTransactionResponse($hParam);
    is_deeply($hHandledResponse, $hExpectedHandledResponse, "$SubTestName - Response handled correctly");
}

sub test_HandleInitializeErrorResponse_WithoutError {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hParam = {};

    my $hExpectedHandledResponse = {
        HasErrorsInResponse => 0,
        Transaction         => {
            ExternalStatus => undef,
        }
    };

    my $hHandledResponse = HandleTransactionResponse($hParam);
    is_deeply($hHandledResponse, $hExpectedHandledResponse, "$SubTestName - Response handled correctly");
}


sub test_HandleAssertErrorResponse_WithError {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hParam = {
        ErrorName   => 'GENERAL_DECLINED',
        TransactionId => '123'
    };

    my $hExpectedHandledResponse = {
        ErrorName => 'GENERAL_DECLINED',
        HasErrorsInResponse => 1,
        TransactionId => '123',
        Transaction         => {
            Id => '123',
            Status => 'FAILED',
            ExternalStatus => 'GENERAL_DECLINED',
        }
    };

    my $hHandledResponse = HandleTransactionResponse($hParam);
    is_deeply($hHandledResponse, $hExpectedHandledResponse, "$SubTestName - Response handled correctly");
}


sub test_HandleAssertErrorResponse_WithoutError {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hParam = {
        Transaction => {
            Id     => '123',
            Status => 'AUTHORIZED'
        }
    };

    my $hExpectedHandledResponse = {
        HasErrorsInResponse => 0,
        Transaction         => {
            Id     => '123',
            Status => 'AUTHORIZED',
            ExternalStatus => 'AUTHORIZED',
        }
    };

    my $hHandledResponse = HandleTransactionResponse($hParam);
    is_deeply($hHandledResponse, $hExpectedHandledResponse, "$SubTestName - Response handled correctly");
}

sub test_FormatFloatForSaferpay {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::Saferpay2::API::Tools');

    $MockifySut->mockImported('DE_EPAGES::Object::API::Table::Currency', 'GetInfoCurrency')->when(String())->thenReturn({ MinorUnit=>2 });

    my $Amount = sprintf("%.2f", rand(1000000));
    my $Currencyid = 'EUR';
    my $ExpectedResult;
    my $SUT = $MockifySut->getMockObject();

    is($SUT->can("FormatFloatForSaferpay")->(''.$Amount, $Currencyid), ''.($ExpectedResult = $Amount * 100), "$SubTestName - in was $Amount, out was $ExpectedResult,  Format float is ok.");

    $Amount = 0.00;
    is($SUT->can("FormatFloatForSaferpay")->(''.$Amount, $Currencyid), ''.($ExpectedResult = $Amount * 100), "$SubTestName - in was $Amount, out was $ExpectedResult,  Format float is ok.");

    $Amount = 99999999.99;
    is($SUT->can("FormatFloatForSaferpay")->(''.$Amount, $Currencyid), ''.($ExpectedResult = $Amount * 100), "$SubTestName - in was $Amount, out was $ExpectedResult,  Format float is ok.");
}

__PACKAGE__->runScript();
