#==============================================================================
# package      DE_EPAGES::Tax::API::Constants
# state        private
#------------------------------------------------------------------------------
# description  Some constants used for CSV export/import and shop wizard tax matrix switch
#==============================================================================
package DE_EPAGES::Tax::API::Constants;
use base Exporter;

use strict;
our @EXPORT = qw(
    @CSV_WARNING_CODES_TAX
    $SHOPWIZARD_TAXMATRIX_MAPPING
    NO_SPLIT_SHIPPING_TAX
    NO_SPLIT_PAYMENT_TAX

);
use vars qw ( @CSV_WARNING_CODES_TAX
              $SHOPWIZARD_TAXMATRIX_MAPPING
            );


# possible warnings during CSV import
@CSV_WARNING_CODES_TAX = qw(
    VOID_FIELD
    NO_MAND_VALUE
);

#========================================================================================
# global       $SHOPWIZARD_TAXMATRIX_MAPPING | mappings for english name of country to TaxMatrix postfix
#               If the ShopWizard is finished we change the TaxMatrix by the address country
#               english name. Here we can define a mapping for countries where the postfix is
#               not the country english name. e.g. "Czech Republic" is mapped to Czechia
#               because we will use the TaxMatrixCzechia for the shop.
#               You can also define different defaults for countries when the default Store tax matrix
#               should not be used. 
#========================================================================================
$SHOPWIZARD_TAXMATRIX_MAPPING = {
    'Czech Republic' => 'Czechia',
};

#========================================================================================
# global       NO_SPLIT_SHIPPING_TAX | list of countries where the shipping methods tax
#                                       is independend from the product tax
#                                       e.g. product 7% tax and product with 19% tax:
#                                       shipping tax is not splitted in 7% and 19%
#                                       the tax from the shipping method is used
#                       | list of country Code2 codes
#========================================================================================
use constant NO_SPLIT_SHIPPING_TAX => qw( FR GB ES IT PT AU NZ );

#========================================================================================
# global       NO_SPLIT_PAYMENT_TAX | list of countries where the payment methods tax
#                                       is independend from the product tax
#                                       e.g. product 7% tax and product with 19% tax:
#                                       payment tax is not splitted in 7% and 19%
#                                       the tax from the payment method is used
#                       | list of country Code2 codes
#========================================================================================
use constant NO_SPLIT_PAYMENT_TAX => qw( FR GB ES IT PT AU NZ );


1;

